/*
 * Project name:
     Soft_I2C demo (Demonstration on working with the RTC Module and Software I2C routines)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Description:
      This project is a simple demonstration how to read date and time on PCF8583
      RTC (real-time clock); the results can be viewed via UART terminal.
 * Test configuration:
      MCU:             FT900
                       http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
      Dev.Board:       Clicker 2 for FT900
                       http://www.mikroe.com/ft90x/clicker-2/
      Oscillator:      CPU at full system clock, 100.000MHz
      Ext. Modules:    UART click - ac:UART_Click
                       RTC click  - ac:RTC_Click
      SW:              mikroC PRO for FT90x
                       http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - Place RTC Click into socket 1.
     - Place USB UART click into socket 2
     - Watch results over USART terminal
 */

#include <stdint.h>

typedef enum LineEnding
{
  WIN, // CRFL
  UNIX //   LF
} LineEnding;

// Global date/time variables
uint8_t
  seconds,
  minutes,
  hours,
  day,
  month,
  year,
  txt[6];

LineEnding
  linEnd = WIN;

// Software I2C connections
sbit Soft_I2C_Scl at GPIO_PIN44_bit;
sbit Soft_I2C_Sda at GPIO_PIN45_bit;

/**
 * Sends the line along with its terminating characters
 * @param text     pointer to string to be sent
 * @param linEnd   choose the line ending either for WIN or UNIX
 */
void UART1_Write_Line(uint8_t* text,
                    LineEnding linEnd)
{
  if (linEnd == WIN)
  {
    UART1_Write_Text(text);
    UART1_Write(13);
    UART1_Write(10);
  }
  else
  {
    UART1_Write_Text(text);
    UART1_Write(10);
  }
}

/**
 * Initialize Soft_I2C and UART
 */
void Init_MCU()
{
  Soft_I2C_Init();   // Software I2C initialization
  UART1_Init(56000); // UART initialization
  UART1_Write_Line("Software I2C test", WIN);
  UART1_Write_Line("Start", WIN);
}

/**
 * Reads time and date information from RTC (PCF8583)
 */
void Read_Time()
{
  Soft_I2C_Start();               // Issue start signal
  Soft_I2C_Write(0xA0);           // Address PCF8583, see PCF8583 datasheet
  Soft_I2C_Write(2);              // Start from address 2
  Soft_I2C_Start();               // Issue repeated start signal
  Soft_I2C_Write(0xA1);           // Address PCF8583 for reading R/W=1

  seconds = Soft_I2C_Read(1);     // Read seconds byte
  minutes = Soft_I2C_Read(1);     // Read minutes byte
  hours   = Soft_I2C_Read(1);     // Read hours byte
  day     = Soft_I2C_Read(1);     // Read year/day byte
  month   = Soft_I2C_Read(0);     // Read weekday/month byte
  Soft_I2C_Stop();                // Issue stop signal
}

void Write_Time()
{
   UART1_Write_Text("Hours:");
   ByteToStr(hours, txt);
   UART1_Write_Text(txt);

   UART1_Write_Text(", minutes :");
   ByteToStr(minutes, txt);
   UART1_Write_Text(txt);

   UART1_Write_Text(", seconds :");
   ByteToStr(seconds, txt);
   UART1_Write_Text(txt);

   UART1_Write_Text(" | ");

   UART1_Write_Text("Day:");
   ByteToStr(day, txt);
   UART1_Write_Text(txt);

   UART1_Write_Text(", month :");
   ByteToStr(month, txt);
   UART1_Write_Text(txt);

   UART1_Write_Text(", year :");
   WordToStr(year+2011, txt);     // the year starts from 2011
   UART1_Write_Line(txt, linEnd);
}

/**
 * Formats date and time
 */
void Transform_Time()
{
  seconds  =  ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);  // Transform seconds
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);  // Transform months
  hours    =  ((hours   & 0xF0) >> 4)*10 + (hours   & 0x0F);  // Transform hours
  year     =   (day     & 0xC0) >> 6;                         // Transform year
  day      =  ((day     & 0x30) >> 4)*10 + (day     & 0x0F);  // Transform day
  month    =  ((month   & 0x10) >> 4)*10 + (month   & 0x0F);  // Transform month
}

/**
 * loops and reads time and sends it to UART
 */
void main()
{
  Init_MCU();
  Delay_ms(100);

  while (1)
  {                          // Endless loop
    Read_Time();             // Read time from RTC(PCF8583)
    Transform_Time();        // Format date and time
    Write_Time();

    Delay_ms(1000);          // Wait 1 second
  }
}