
#include "Camera_Demo.h"
#include "Camera_Demo_objects.h"

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Line data of frame.
// Set address of camera_frameLinePixels to be aligned with 4 (function call DCAP_ReadFIFO will be faster).
static char camera_frameLinePixels[_CAMERA_HOR_RES * _CAMERA_BYTES_PER_PIXEL] absolute 0x10;

// Currnet values for red and blue channel gain.
static const _RED_CHANNEL_GAIN  = 0x38;
static const _BLUE_CHANNEL_GAIN = 0x66;
static char currRedChannelGain, currBlueChannelGain;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Camera Configuration for QVGA (320x240) format.

typedef struct
{
    char id;
    char val;
}
Camera_CfgQVGA_t;


static const Camera_CfgQVGA_t CAMERA_CFG_QVGA[] =
{
        {REG_COM15,  COM15_RGB565},
        {REG_CLKRC,  0x5},

        // Gamma values
        {0x7a, 0x20},
        {0x7b, 0x1c},
        {0x7c, 0x28},
        {0x7d, 0x3c},
        {0x7e, 0x55},
        {0x7f, 0x68},
        {0x80, 0x76},
        {0x81, 0x80},
        {0x82, 0x88},
        {0x83, 0x8f},
        {0x84, 0x96},
        {0x85, 0xa3},
        {0x86, 0xaf},
        {0x87, 0xc4},
        {0x88, 0xd7},
        {0x89, 0xe8},

        // Reserved
        {0x21, 0x02},
        {0x22, 0x91},
        {0x29, 0x07},
        {0x33, 0x0b},
        {0x35, 0x0b},
        {0x37, 0x1d},
        {0x38, 0x71},
        {0x39, 0x2a},
        {0x3c, 0x78},
        {0x4d, 0x40},
        {0x4e, 0x20},
        {0x69, 0x0c},                           // Fixed gain control
        {0x6b, 0x40},                           // PLL clock x4
        {0x74, 0x19},
        {0x8d, 0x4f},
        {0x8e, 0x00},
        {0x8f, 0x00},
        {0x90, 0x00},
        {0x91, 0x00},
        {0x92, 0x00},
        {0x96, 0x00},
        {0x9a, 0x80},
        {0xb0, 0x84},
        {0xb1, 0x0c},
        {0xb2, 0x0e},
        {0xb3, 0x82},
        {0xb8, 0x0a},

        // AWB
        {0x43, 0x14},
        {0x44, 0xf0},
        {0x45, 0x34},
        {0x46, 0x58},
        {0x47, 0x28},
        {0x48, 0x3a},
        {0x59, 0x88},
        {0x5a, 0x88},
        {0x5b, 0x44},
        {0x5c, 0x67},
        {0x5d, 0x49},
        {0x5e, 0x0e},

        // Lens correction
        {0x64, 0x04},
        {0x65, 0x20},
        {0x66, 0x05},
        {0x94, 0x04},
        {0x95, 0x08},
        {0x6c, 0x0a},
        {0x6d, 0x55},
        {0x6e, 0x11},
        {0x6f, 0x9f},
        {0x6a, 0x40},
        {0x13, 0xe7},
        {0x15, 0x00},

        // Matrix coefficient
        {0x4f, 0x80},                           // Matrix coefficient
        {0x50, 0x80},                           // Matrix coefficient
        {0x51, 0x00},
        {0x52, 0x22},
        {0x53, 0x5e},
        {0x54, 0x80},
        {0x58, 0x9e},

        {0x41, 0x08},
        {0x3f, 0x00},                           // Edge enhancement
        {0x75, 0x05},                           // Edge enhancement
        {0x76, 0xe1},                           // Edge enhancement, white/black pixel correction
        {0x4c, 0x00},                           // Denoise
        {0x77, 0x01},                           // Denoise
        {0x3d, 0xc2},
        {0x4b, 0x09},
        {0xc9, 0x60},                           // Saturation
        {0x41, 0x38},
        {0x56, 0x40},
        {0x34, 0x11},
        {0x3b, 0x02},
        {0xa4, 0x89},
        {0x96, 0x00},
        {0x97, 0x30},
        {0x98, 0x20},
        {0x99, 0x30},
        {0x9a, 0x84},
        {0x9b, 0x29},
        {0x9c, 0x03},
        {0x9d, 0x4c},                           // Banding filter
        {0x9e, 0x3f},                           // Banding filter
        {0x78, 0x04},
        {0x79, 0x01},
        {0xc8, 0xf0},
        {0x79, 0x0f},
        {0xc8, 0x00},
        {0x79, 0x10},
        {0xc8, 0x7e},
        {0x79, 0x0a},
        {0xc8, 0x80},
        {0x79, 0x0b},
        {0xc8, 0x01},
        {0x79, 0x0c},
        {0xc8, 0x0f},
        {0x79, 0x0d},
        {0xc8, 0x20},
        {0x79, 0x09},
        {0xc8, 0x80},
        {0x79, 0x02},
        {0xc8, 0xc0},
        {0x79, 0x03},
        {0xc8, 0x40},
        {0x79, 0x05},
        {0xc8, 0x30},
        {0x79, 0x26},
        {0x09, 0x03},                           // Output drive
        {0x55, 0x82},                           // Brightness
        {0x56, 0x50},                           // Contrast
        {0x3b, 0x42},                           // Night mode
        {0x1e, 0x23},                           // Mirror, vflip, black sun was 0x07
        {REG_HREF,   0x80},
        {REG_HSTART, 0x16},
        {REG_HSTOP,  0x08},

        /* AGC and AEC parameters.  Note we start by disabling those features,
        then turn them only after tweaking the values. */
        {REG_COM8, COM8_FASTAEC | COM8_AECSTEP | COM8_BFILT},
        {REG_GAIN, 0x80},
        {REG_AECH, 0},
        {REG_COM4, 0x40},
        {REG_COM9, 0x18},                       // 4x gain + magic rsvd bit
        {REG_BD50MAX, 0x05},
        {REG_BD60MAX, 0x07},
        {REG_AEW, 0x95},
        {REG_AEB, 0x33},
        {REG_VPT, 0xe3},
        {REG_HAECC1, 0x78},
        {REG_HAECC2, 0x68},
        {0xa1, 0x03},
        {REG_HAECC3, 0xd8},
        {REG_HAECC4, 0xd8},
        {REG_HAECC5, 0xf0},
        {REG_HAECC6, 0x90},
        {REG_HAECC7, 0x94},
        {REG_BLUE, _BLUE_CHANNEL_GAIN},
        {REG_RED, _RED_CHANNEL_GAIN},
        {REG_COM8, COM8_FASTAEC | COM8_AECSTEP | COM8_BFILT | COM8_AGC | COM8_AEC},

        {0xff, 0xff},
};
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Camera module connections.
sfr sbit Camera_PowerDown_bit at GPIO_PIN18_bit;
sfr sbit Camera_Reset_bit     at GPIO_PIN50_bit;


// Pins for setting colors.
sfr sbit Camera_RedColorUP    at GPIO_PIN1_bit;
sfr sbit Camera_RedColorDown  at GPIO_PIN0_bit;

sfr sbit Camera_BlueColorUP   at GPIO_PIN5_bit;
sfr sbit Camera_BlueColorDown at GPIO_PIN4_bit;

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief Declarations of private functions */

static char Camera_WriteReg(char reg, char regVal);
static char Camera_ReadReg(char reg, char * regVal);
static char Camera_Reset(void);
static char Camera_ConfigRegisters(const Camera_CfgQVGA_t *configuration);
static void Camera_SetColor(void);

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   Inits Data Capture Interface module, I2C interface and registers of OV7670 camera.
 *
 * @param   none
 *
 * @return  _CAMERA_STATUS_OK if operation is successful, otherwise _CAMERA_STATUS_FAIL.
 */
 
char Camera_InitQVGA(void)
{
    char camID, statusFlag;

    // Config pins for setting colors.
    GPIO_Pin_Digital_Input(0);
    GPIO_Pin_Digital_Input(1);
    GPIO_Pin_Digital_Input(4);
    GPIO_Pin_Digital_Input(5);
    
    // Set default values for red i blue gains.
    currRedChannelGain  = _RED_CHANNEL_GAIN;
    currBlueChannelGain = _BLUE_CHANNEL_GAIN;

    // Put camera in normal mode.
    Camera_PowerDown_bit = 0;
    GPIO_Pin_Digital_Output(18);
    Camera_Reset_bit = 1;
    GPIO_Pin_Digital_Output(50);

    DCAP_Init();                                                                // Enable camera module clock and config corresponding pins.
    DCAP_SetTrigpat(2);                                                         // Set data capture trigger position.
    DCAP_SetCount(_CAMERA_HOR_RES * 2);                                         // Set number of bytes to capture after the last trigger event.
    DCAP_SetThreshold(_CAMERA_HOR_RES * 2);                                     // Set the threshold value.
    DCAP_SetClockPolarity(1);                                                   // Capture data on clock rising edge.

    I2CM1_Init(_I2CM_SPEED_MODE_FAST, _I2CM_SWAP_DISABLE);                      // Init I2C interface.
    
    statusFlag = Camera_ReadReg(0xA, &camID);                                   // Read PID register (product ID number). Shoult be 0x76.

    // Check statusFlag and PID
    if( (statusFlag != _CAMERA_STATUS_OK) ||
        (camID != _CAMERA_ID)
      )
    {
        return _CAMERA_STATUS_FAIL;
    }

    if(Camera_Reset() != _CAMERA_STATUS_OK)                                     // Do camera soft reset.
    {
        return _CAMERA_STATUS_FAIL;
    }

    Delay_ms(100);


    if(Camera_WriteReg(REG_COM7, COM7_FMT_QVGA | COM7_RGB) != _CAMERA_STATUS_OK)// Set output format to QVGA with RGB.
    {
        return _CAMERA_STATUS_FAIL;
    }


    if(Camera_configRegisters(&CAMERA_CFG_QVGA[0]) != _CAMERA_STATUS_OK)        // Config rest of camera properties.
    {
        return _CAMERA_STATUS_FAIL;
    }

    return _CAMERA_STATUS_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   Waits vertical synchronization signal.
 *
 * @param   none
 *
 * @return  none
 */
 
void Camera_WaitVSync(void)
{
  GPIO_Pin_Config(_CAMERA_CAM_VD_PIN, _GPIO_DIR_INPUT, __mE_GPIO_CFG_DEFAULT);  // Config VD pin to be GPIO input.

  while (GPIO_Pin_Read(_CAMERA_CAM_VD_PIN) == 0) ;                              // Wait VD rising edge.
  
  while (GPIO_Pin_Read(_CAMERA_CAM_VD_PIN) != 0) ;                              // Wait VD falling edge.

  GPIO_Pin_Config(_CAMERA_CAM_VD_PIN, _GPIO_DIR_NO_CHANGE, __mE_GPIO_CFG_DEFAULT | _GPIO_CFG_SPEC_FUNC_1); // Config VD pin to Special function 1.
  DCAP_drain();                                                                 // Empties FIFO buffer.
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function should be called after start of frame detected. 
 *          It process image line by line.
 *
 * @param   none
 *
 * @return  none
 */
 
char * Camera_GetFrameLine(void)
{
    while(DCAP_HasData() != 1);

    DCAP_ReadFIFO(camera_frameLinePixels, _CAMERA_HOR_RES * 2);                 // Read line data.
    
    return camera_frameLinePixels;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function used for drawing image, and set color gain values.
 *
 * @param   none
 *
 * @return  none
 */
 
void Camera_ProcessImage(void)
{
    DrawImage(&Image1);
    Camera_SetColor();
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function sets red and blue chanel gains.
 *
 * @param   none
 *
 * @return  none
 */

static void Camera_SetColor(void)
{
    if((Camera_RedColorUP == 0) && (currRedChannelGain < 255))
    {
        Camera_WriteReg(REG_RED, ++currRedChannelGain);
    }
    else if((Camera_RedColorDown == 0) && (currRedChannelGain > 0))
    {
        Camera_WriteReg(REG_RED, --currRedChannelGain);
    }
    
    if((Camera_BlueColorUP == 0) && (currBlueChannelGain < 255))
    {
        Camera_WriteReg(REG_BLUE, ++currBlueChannelGain);
    }
    else if((Camera_BlueColorDown == 0) && (currBlueChannelGain > 0))
    {
        Camera_WriteReg(REG_BLUE, --currBlueChannelGain);
    }
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function writes one byte to OV7670 Device Control Register.
 *
 * @param   reg    - Address of register.
 * @param   regVal - Value which will be written.
 *
 * @return  _CAMERA_STATUS_OK if operation is successful, otherwise _CAMERA_STATUS_FAIL.
 */
 
static char Camera_WriteReg(char reg, char regVal)
{
    char dat[2];
    char err;
    
    dat[0] = reg;
    dat[1] = regVal;

    I2CM_Set_Slave_Address(OMNICAM_IIC_ADDR);
    err = I2CM_Write_Bytes(dat, 2);
    if(err)
    {
        return _CAMERA_STATUS_FAIL;
    }

    Delay_us(1000);
    return _CAMERA_STATUS_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function reads OV7670 Device Control Register.
 *
 * @param   reg    - Address of register.
 * @param   regVal - Value of corresponding register
 *
 * @return  _CAMERA_STATUS_OK if operation is successful, otherwise _CAMERA_STATUS_FAIL.
 */
 
static char Camera_ReadReg(char reg, char * regVal)
{
    char err;

    I2CM_Set_Slave_Address(OMNICAM_IIC_ADDR);
    Delay_500us();
    err = I2CM_Write(reg);
    if(err)
    {
        return _CAMERA_STATUS_FAIL;
    }

    err = I2CM_Read(regVal);
    if(err)
    {
        return _CAMERA_STATUS_FAIL;
    }

    Delay_us(1000);
    return _CAMERA_STATUS_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function resets all registers to default values.
 *
 * @param   none
 *
 * @return  _CAMERA_STATUS_OK if operation is successful, otherwise _CAMERA_STATUS_FAIL.
 */
 
static char Camera_Reset(void)
{
    char statusFlag;
    statusFlag = Camera_WriteReg(0x12, 0x80) ;
    Delay_ms(200);
    return statusFlag;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
/**@brief   This function writes configuration to Device Control Registers.
 *
 * @param   configuration - Camera Configuration for QVGA.
 *
 * @return  _CAMERA_STATUS_OK if operation is successful, otherwise _CAMERA_STATUS_FAIL.
 */
 
static char Camera_ConfigRegisters(const Camera_CfgQVGA_t *configuration)
{
    int i;
    char err;
    for (i=0; ; i++)
    {
        if (configuration[i].id == 0xff && configuration[i].val == 0xff)
        {
            break;
        }

        if(Camera_WriteReg(configuration[i].id, configuration[i].val) != _CAMERA_STATUS_OK)
        {
            return _CAMERA_STATUS_FAIL;
        }
        Delay_ms(1);
        
    }
    return _CAMERA_STATUS_OK;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////