/* Project name:
     Ethernet Demo (Ethernet Library demo for internal Ethernet module)
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release;
 * description  :
 * this code shows how to use the Ethernet library:
 * - the board will reply to ARP & ICMP echo requests;
 * - the board will reply to UDP requests on any port -- returns the request
 *   in upper char with a header made of remote host IP & port number;
 * - the board will reply to HTTP requests on port 80, GET method with pathnames:
 *   /               will return the HTML main page
 *   /s              will return board status as text string
 *   /t0 ... /t7     will toggle RE8 to RE15 bit and return HTML main page
 *                   all other requests return also HTML main page
 *
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.board:       EasyFT90x v7 - ac:ETHERNET
                      http://www.mikroe.com/easyft90x
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    None.
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
 * NOTES:
     - prepare the buttons for testing: pull-down PORT 24-31
       and put SW1.4 in VCC position (board specific);
     - prepare analog input reading: place jumper on GPIO 06 at J81;
     - prepare LEDs for testing: turn ON PORT 32-39 LEDs at SW8.5.
 */

#include        "__Ethernet_Intern.h"

/**
 * Constant strings
 */
unsigned char
  httpHeader[]         = "HTTP/1.1 200 OK\nContent-type: ",  // HTTP header
  httpMimeTypeHTML[]   = "text/html\n\n",                    // HTML MIME type
  httpMimeTypeScript[] = "text/plain\n\n",                   // TEXT MIME type
  httpMethod[]         = "GET /";                            // HTTP method to recognize

/**
 * web page, splited into 2 parts :
 * when coming short of flash, fragmented data is handled more efficiently by linker
 * this HTML page calls the boards to get its status, and builds itself with javascript
 */

// Change the IP address of the page to be refreshed
char indexPage[] =
"<meta http-equiv=\"refresh\" content=\"2;url=http://10.101.14.52\">\
<HTML><HEAD></HEAD><BODY style=\"background-color:#DEDEDE;\">\
<h1 style=\"text-align: center; margin-bottom: 0px; font-family: Arial\"><span style = \"color: #B1CF82;\">Easy<span style=\"color: #dba722;\">FT90x</span></span></h1>\
<h2 style=\"text-align: center; margin-top: 0px; font-family: tahoma\">mini-web server</h2>\
<script src=s></script>\
<table align=\"center\"><tr><td valign=top><table border=1 style=\"font-size:20px ;font-family: terminal ;\">\
<tr><th colspan=2>ADC</th></tr>\
<tr><td>AN0</td><td><script>document.write(ADC)</script></td></tr>\
</table></td><td><table border=1 style=\"font-size:20px ;font-family: terminal ;\">\
<tr><th colspan=2>GPIO 24-31</th></tr>\
<script>\
var str,i;\
str=\"\";\
for(i=0;i<8;i++)\
{str+=\"<tr><td>BUTTON #\"+i+\"</td>\";\
if(BTNS&(1<<i)){str+=\"<td bgcolor=green>ON\";}\
else {str+=\"<td>OFF\";}\
str+=\"</td></tr>\";}\
document.write(str);\
</script>\
";

char indexPage2[] =  "</table></td><td>\
<table border=1 style=\"font-size:20px ;font-family: terminal ;\">\
<tr><th colspan=3>GPIO 32-39</th></tr>\
<script>\
var str,i;\
str=\"\";\
for(i=0;i<8;i++)\
{str+=\"<tr><td>LED #\"+i+\"</td>\";\
if(LEDS&(1<<i)){str+=\"<td bgcolor=green>ON\";}\
else {str+=\"<td>OFF\";}\
str+=\"</td><td><a href=/t\"+i+\">toggle</a></td></tr>\";}\
document.write(str) ;\
</script>\
</table></td></tr></table>\
<center>This is HTTP request #<script>document.write(REQ)</script><\center>\
</BODY></HTML>\
";


char myMacAddr[6]    = {0x00, 0x14, 0xA5, 0x74, 0x19, 0x3f};          // my MAC address
char myIpAddr[4]     = {10, 101,  14, 52};                            // my IP address
char gwIpAddr[4]     = {10, 101,  14, 51};                            // gateway (router) IP address
char ipMask[4]       = {255, 255, 254, 0};                            // network mask
char dnsIpAddr[4]    = {10, 101,  14, 99};                            // DNS server IP address

unsigned char   getRequest[15];                                        // HTTP request buffer
unsigned char   dyna[31] ;                                             // buffer for dynamic response
unsigned long   httpCounter = 0;                                       // counter of HTTP requests

unsigned int    Ethernet_Intern_UserTCP(unsigned char* remoteHost,
                                         unsigned int  remotePort,
                                         unsigned int localPort,
                                         unsigned int reqLength,
                                  TEthInternPktFlags* flags)
{
    unsigned int    len;            // my reply length
    unsigned int    offset = 0;     // possition in payload buffer

    // should we close  socket after response is sent?
    // library closes tcp socket by default if canCloseTCP flag is not reset here
    // flags->canCloseTCP = 0; // 0         - do not close socket
                               // otherwise - close socket

    if(localPort != 80)        // listening only web request to port 80
    {
        return 0;
    }

    // get 10 first bytes of the request
    for(len = 0; len < 10; len++)
    {
        getRequest[len] = Ethernet_Intern_readPayloadByte(offset++);
    }
    getRequest[len] = 0;

    if(memcmp(getRequest, httpMethod, 5))       // check if GET method is sent (if not, exit)
    {
        return 0;
    }
    httpCounter++;                              // one more request done

    offset = 0;

    // if request path name starts with s (from the script call at the beginning of the page), send dynamic data
    if(getRequest[5] == 's')
    {
        // the text string sent in reply by this request is interpreted as javascript statements by browsers
        offset += Ethernet_Intern_writePayloadString(httpHeader, offset);         // HTTP header
        offset += Ethernet_Intern_writePayloadString(httpMimeTypeScript, offset); // with text MIME type

        // add ADC1 value to reply
        WordToStr(ADC_Get_Sample(1), dyna);
        offset += Ethernet_Intern_writePayloadString("var ADC=", offset);
        offset += Ethernet_Intern_writePayloadString(dyna, offset);
        offset += Ethernet_Intern_writePayloadString("; ", offset);

        // add GPIO PORT 24-31 value (buttons) to the reply
        offset += Ethernet_Intern_writePayloadString("var BTNS=", offset);
        WordToStr(GPIO_PORT_24_31, dyna);
        offset += Ethernet_Intern_writePayloadString(dyna, offset);
        offset += Ethernet_Intern_writePayloadString("; ", offset);

        // add GPIO PORT 32-39 value (LEDs) to reply
        offset += Ethernet_Intern_writePayloadString("var LEDS=", offset);
        WordToStr(GPIO_PORT_32_39, dyna);
        offset += Ethernet_Intern_writePayloadString(dyna, offset);
        offset += Ethernet_Intern_writePayloadString("; ", offset);

        // add HTTP requests counter to reply
        WordToStr(httpCounter, dyna);
        offset += Ethernet_Intern_writePayloadString("var REQ=", offset);
        offset += Ethernet_Intern_writePayloadString(dyna, offset);
        offset += Ethernet_Intern_writePayloadString("; ", offset);
    }
    else if (getRequest[5] == 't')                    // if request path name starts with t, toggle PORTJ (LED) bit number that comes after
    {
        unsigned long
          bitMask = 0;                                // for bit mask

        if (isdigit(getRequest[6]))                   // if bit number belongs to [0..9], bits 8 & 9 does not exist but does not matter
        {
            bitMask = getRequest[6] - '0';            // convert ASCII to integer
            bitMask = 1 << bitMask;                   // create bit mask
            GPIO_PORT_32_39 ^= bitMask;               // toggle GPIO PORT 32-39
        }
    }

    // what do to by default
    if (offset == 0)
    {
        offset += Ethernet_Intern_writePayloadString(httpHeader,       offset); // HTTP header
        offset += Ethernet_Intern_writePayloadString(httpMimeTypeHTML, offset); // with HTML MIME type
        offset += Ethernet_Intern_writePayloadString(indexPage,        offset); // HTML page first part
        offset += Ethernet_Intern_writePayloadString(indexPage2,       offset); // HTML page second part
    }

    // return to the library with the number of bytes to transmit
    return offset;

}

unsigned int    Ethernet_Intern_UserUDP(unsigned char* remoteHost,
                                         unsigned int  remotePort,
                                         unsigned int localPort,
                                         unsigned int reqLength,
                                  TEthInternPktFlags* flags)
{
    unsigned int    len;                           // my reply length
    unsigned int    offset = 0;                    // possition in payload buffer
    unsigned int    i = 0;
    unsigned char   bufferLoc[20];                 // we want receive packet with payload up to 20 bytes

    if (reqLength > 20)
      return 0;

    // Read "reqLength" bytes from payload buffer (starting from position "offset") into local buffer "bufferLoc"
    Ethernet_Intern_readPayloadBytes(bufferLoc, offset, reqLength);

    // reply is made of the remote host IP address in human readable format
    ByteToStr(remoteHost[0], dyna);                // first IP address byte
    dyna[3] = '.';
    ByteToStr(remoteHost[1], dyna + 4);            // second
    dyna[7] = '.';
    ByteToStr(remoteHost[2], dyna + 8);            // third
    dyna[11] = '.';
    ByteToStr(remoteHost[3], dyna + 12);           // fourth

    dyna[15] = ':';                                // add separator

    // then remote host port number
    WordToStr(remotePort, dyna + 16);
    dyna[21] = '[';
    WordToStr(localPort, dyna + 22);
    dyna[27] = ']';
    dyna[28] = 0;

    // the total length of the request is the length of the dynamic string plus the text of the request
    len = reqLength + 28;

    // put the dynamic string into the transmit buffer
    Ethernet_Intern_writePayloadBytes(dyna, offset, 28);
    offset += 28;

    // afterwards, put the request string converted into upper char into the transmit buffer
    while (reqLength--)
    {
      Ethernet_Intern_writePayloadByte(toupper(bufferLoc[i++]), offset++);
    }

    // back to the library with the length of the UDP reply
    return len;
}

void main()
{
  GPIO_Digital_Input (&GPIO_PORT_24_31, _GPIO_PINMASK_ALL);
  GPIO_Digital_Output(&GPIO_PORT_32_39, _GPIO_PINMASK_ALL);

  ADC_Init();
  ADC_Set_Input_Channel(_ADC_CHANNEL_1);

  Ethernet_Intern_Init(myMacAddr, myIpAddr, _ETHERNET_CFG_SPD100 | _ETHERNET_CFG_FULLDUPLEX);
  Ethernet_Intern_confNetwork(ipMask, gwIpAddr, dnsIpAddr);

  while (1)
  {
    // process incoming Ethernet packets
    Ethernet_Intern_doPacket(); // if necessary, test the return value to get error code
    /**
     * add your stuff here if needed
     * Ethernet_Intern_doPacket() must be called as often as possible
     * otherwise, packets could be lost
     */
  }
}