/* Project name:
     Serial Flash
 * Copyright:
     (c) Mikroelektronika, 2015.
 * Revision History:
       - initial release;
 * Description:
     This module contains a set of functions that are used for communication with Serial Flash.
     Serial Flash use SPI bus to communicate with MCU.
 * Test configuration:
     Device:          FT900
                      http://www.ftdichip.com/Support/Documents/DataSheets/ICs/DS_FT900_1_2_3.pdf
     Dev.Board:       EasyFT90x v7 - ac:FLASH
                      http://www.mikroe.com/easyft90x
                      
     Oscillator:      CPU at full system clock, 100.000MHz
     Ext. Modules:    EasyTFT - ac:EasyTFT
     
     SW:              mikroC PRO for FT90x
                      http://www.mikroe.com/mikroc/ft90x/
                      
 * NOTES:
     - Turn on SPI lines at SW2.6, SW2.7, sw2.8 and Serial Flash CS at SW2.
 */

#define _DATA_ARRAY_SIZE 16
#include <built_in.h>
#include "Serial_Flash.h"
#include "resources.h"

// TFT module connections
char TFT_DataPort at GPIO_PORT_32_39;
sbit TFT_RST at GPIO_PIN42_bit;
sbit TFT_RS at GPIO_PIN40_bit;
sbit TFT_CS at GPIO_PIN41_bit;
sbit TFT_RD at GPIO_PIN46_bit;
sbit TFT_WR at GPIO_PIN47_bit;
sbit TFT_BLED at GPIO_PIN58_bit;
// End TFT module connections

/**************************************************************************************************
* Serial Flash Chip Select connection
**************************************************************************************************/

char cSF_test_status;

void Auxiliary_Code() {
  // Place your code here
  TFT_WR = 0;
  asm nop;
  TFT_WR = 1;
}

void Set_Index(unsigned short index) {
  // Place your code here
  TFT_RS = 0;
  asm nop;
  TFT_DataPort = index;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Command(unsigned short cmd) {
  // Place your code here
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = cmd;
  asm nop; asm nop;
  Auxiliary_Code();
}

void Write_Data(unsigned int _data) {
  // Place your code here
  TFT_RS = 1;
  asm nop;
  TFT_DataPort = Hi(_data);
  asm nop; asm nop;
  Auxiliary_Code();
  asm nop;
  TFT_DataPort = Lo(_data);
  asm nop; asm nop;
  Auxiliary_Code();
}

/*************************************************************************************************
* Init MCU function
**************************************************************************************************/
void InitMCU(){
// SPI config
  GPIO_Digital_Output(&GPIO_PORT_48_55, _GPIO_PINMASK_3);
}
/*********************************************************************************************/
void DrawSFScr(){
  TFT_Set_Active(Set_Index, Write_Command, Write_Data);
  TFT_Init_ILI9341_8bit(320,240);
  TFT_Fill_Screen(CL_WHITE);
  TFT_Set_Pen(CL_Black, 1);
  TFT_Line(20, 220, 300, 220);
  TFT_LIne(20,  46, 300,  46);
  TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("SERIAL  FLASH", 95, 14);
  TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
  TFT_Write_Text("EasyMxv7s", 19, 223);
  TFT_Set_Font(&Verdana12x13_Regular, CL_RED, FO_HORIZONTAL);
  TFT_Write_Text("www.mikroe.com", 200, 223);
  TFT_Set_Font(&TFT_defaultFont, CL_BLACK, FO_HORIZONTAL);
}
/**************************************************************************************************
* Function SF_Start()
* -------------------------------------------------------------------------------------------------
* Overview: Function Initialize SPI bus and serial flash
* Input: Nothing
* Output: Nothing
**************************************************************************************************/
void SF_Start() {
  SPIM1_Init_Advanced(_SPI_MASTER_CLK_RATIO_8,
      _SPI_CFG_POLARITY_IDLE_LOW |
   _SPI_CFG_PHASE_CAPTURE_RISING,
         _SPI_SS_LINE_NONE);
  TFT_Set_Pen(CL_WHITE, 1);
  TFT_BLED = 1;
  TFT_Set_Brush(1, CL_WHITE, 0, 0, 0, 0);
  SerialFlash_init();
  SerialFlash_WriteEnable();
  Delay_ms(500);
}

/**************************************************************************************************
* Function SF_Test()
* -------------------------------------------------------------------------------------------------
* Overview: Function run the serial flash test
* Input: Nothing
* Output: test status: 0 - skiped; 1 - pass; 2 - fail
**************************************************************************************************/
void SF_Test(char *test)
{
  unsigned char temp, SerialFlashID, txt[12];
  unsigned char i, success;

  // Reset error flag
  *test = 0;

  // Erase entire flash
  TFT_Write_Text("1. Erasing chip... ", 90, 65);
  SerialFlash_ChipErase();
  TFT_Write_Text("Done!", 200, 65);

  // Read Flash ID
  SerialFlashID = SerialFlash_ReadID();
  ByteToStr(SerialFlashID, txt);
  TFT_Write_Text("2. Flash ID:", 90, 85);
  TFT_Write_Text(txt, 170, 85);
  Delay_ms(500);

  // Write and read back a single byte
  temp = 221;
  TFT_Write_Text("3. Writting:", 90, 105);
  ByteToStr(temp, txt);
  TFT_Write_Text(txt, 170, 105);
  SerialFlash_WriteByte(temp, 0x123456);

  TFT_Write_Text("4. Reading: ", 90, 125);
  temp = SerialFlash_ReadByte(0x123456);
  ByteToStr(temp, txt);
  TFT_Write_Text(txt, 170, 125);
  Delay_ms(500);

  // Write the repetitive pattern of data to fill the first 4K of memory
  // and then read the entire flash again and check the data correctness
  TFT_Write_Text("5. Writing array...", 90, 145);
  SerialFlash_WriteArray(0x0000, &write_array, _DATA_ARRAY_SIZE);
  Delay_ms(500);

  TFT_Write_Text("6. Reading array...", 90, 165);
  SerialFlash_ReadArray(0x0000, &read_array, _DATA_ARRAY_SIZE);
  Delay_ms(500);
  success = 1;

  for (i = 0; i<_DATA_ARRAY_SIZE; i++)
  {
    if (read_array[i] != write_array[i])
    {
      success = 0;
      break;
    }
  }

  if (success)
  {
    TFT_Write_Text("7. Success - Full match!", 90, 185);
    *test = 1;
  }
  else
  {
    TFT_Write_Text("7. Failed", 90, 185);
    *test = 2;
  }
}

/**************************************************************************************************
* main function
**************************************************************************************************/

void main(){
  InitMCU();
  DrawSFScr();
  SF_Start();
  SF_Test(&cSF_test_status);
}